require 'active_resource'
require 'ranbanery/base'
require 'ranbanery/project'
require 'ranbanery/task'
require 'ranbanery/subtask'
require 'ranbanery/column'
require 'ranbanery/comment'
require 'ranbanery/user'

module Kanbanery
  class Client
    attr_accessor :workspace, :key

    BASE_URL = 'https://%s.kanbanery.com/api/v1'

    def initialize(workspace, key)
      self.workspace = workspace
      self.key       = key
    end

    def resource(name, options={})
      the_workspace = self.workspace
      the_key       = self.key
      klass      = "Kanbanery::#{name.to_s.classify}".constantize
      Class.new(klass) do |c|
        def self.name; superclass.name; end
        c.element_name                    = name.to_s
        c.site                            = "#{BASE_URL % the_workspace}#{options[:prefix]}"
        c.headers['X-Kanbanery-ApiToken'] = the_key
      end
    end

    def projects
      resource(:project)
    end

    def users(project_id=nil)
      resource(:user, prefix: project_id ? "/projects/#{project_id}" : nil)
    end

    def columns(project_id=nil)
      resource(:column, prefix: project_id ? "/projects/#{project_id}" : nil)
    end

    def tasks(project_id=nil)
      resource(:task, prefix: project_id ? "/projects/#{project_id}" : nil)
    end

    def subtasks(task_id=nil)
      resource(:subtask, prefix: task_id ? "/tasks/#{task_id}" : nil)
    end

    def comments(task_id=nil)
      resource(:comment, prefix: task_id ? "/tasks/#{task_id}" : nil)
    end

  end
end
